const exports = {},
    prefixRe = /^[^\:]+\:{2}/,
    defaultPrefix = 'src',
    usedCodes = {};

export function include(name) {

    if (usedCodes.hasOwnProperty(name)) {

        return usedCodes[name];
    }

    if (prefixRe.test(name)) {

        return usedCodes[name] = exports[name];
    }

    return usedCodes[name] = exports[`${defaultPrefix}::${name}`];
}

exports['src::is.empty'] = function(data) {









    /**
     * 
     * 判断给定数据是否为空
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为空，则返回true , 否则返回 false
     * 
     */

    return data === null || data === undefined || data === '' || data.length === 0;
};
exports['src::is.string'] = function(data) {









    /**
     * 
     * 判断给定数据是否字符串
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为字符串，则返回true , 否则返回 false
     * 
     */

    return typeof data === 'string';
};
exports['src::array.from'] = function(data) {



    const isEmpty = include('is.empty');
    const isString = include('is.string');





    /**
     * 
     * 将任何数据转换成可用的数组类型
     * 
     * @import is.empty
     * 
     * @import is.string
     * 
     * @param {*} data 数据
     * 
     * @return {Array}
     * 
     */

    if (isEmpty(data)) {

        return [];
    }

    if (data && data.length !== undefined && !isString(data)) {

        return Array.from(data);

    }

    return [
        data
    ];
};
exports['src::object.copy'] = function(dest, source, keys = []) {









    /**
     * 
     * 拷贝对象
     * 
     * 
     * @param {Objet} dest 目标对象
     * 
     * @param {Object} source 拷贝对象 
     * 
     * @param {Array} [keys = []] 需要拷贝的键名
     * 
     * @return {Object} 附加了拷贝对象指定键名的目标对象
     * 
     */

    for (let key of keys) {

        if (key in source) {

            dest[key] = source[key];
        }
    }

    return dest;
};
exports['src::is.function'] = function(data) {









    /**
     * 
     * 判断给定数据是否为函数
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为函数，则返回true , 否则返回 false
     * 
     */

    return typeof data === 'function';
};
exports['src::is.object.simple'] = function(data) {









    /**
     * 
     * 判断给定数据是否简单对象类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为简单对象类型，则返回true , 否则返回 false
     * 
     */

    return data instanceof Object && data.constructor === Object;
};
exports['src::is.object'] = function(data) {









    /**
     * 
     * 判断给定数据是否对象类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为对象类型，则返回true , 否则返回 false
     * 
     */

    return Object.prototype.toString.call(data) === '[object Object]';
};
exports['src::is.object.simple'] = function(data) {









    /**
     * 
     * 判断给定数据是否简单对象类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为简单对象类型，则返回true , 否则返回 false
     * 
     */

    return data instanceof Object && data.constructor === Object;
};
exports['src::is.function'] = function(data) {









    /**
     * 
     * 判断给定数据是否为函数
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为函数，则返回true , 否则返回 false
     * 
     */

    return typeof data === 'function';
};
exports['src::is.number'] = function(data) {









    /**
     * 
     * 判断给定数据是否数值类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为数值类型，则返回true , 否则返回 false
     * 
     */

    return typeof data === 'number';
};
exports['src::is.boolean'] = function(data) {









    /**
     * 
     * 判断给定数据是否布尔类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为布尔类型，则返回true , 否则返回 false
     * 
     */

    return typeof data === 'number';
};
exports['src::string.from.literal'] = function(data) {



    const isString = include('is.string');
    const isNumber = include('is.number');
    const isBoolean = include('is.boolean');





    /**
     * 
     * 将任何数据类型的数据转换成字面量文本
     * 
     * @import is.string
     * 
     * @import is.number
     * 
     * @import is.boolean
     * 
     * @param {Mixed} data 需要转换的数据
     * 
     * @return {String} 转换后的字面量文本
     * 
     */

    if (isString(data)) {

        return `'${data}'`;
    }

    if (isNumber(data) || isBoolean(data)) {

        return String(data);
    }

    return "''";
};
exports['src::is.array'] = function(data) {









    /**
     * 
     * 判断给定数据是否为数组
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为数组，则返回true , 否则返回 false
     * 
     */

    return Array.isArray(data);
};
exports['src::is.object'] = function(data) {









    /**
     * 
     * 判断给定数据是否对象类型
     * 
     * @param {*} data 数据
     * 
     * @return {Boolean} 如果给定数据为对象类型，则返回true , 否则返回 false
     * 
     */

    return Object.prototype.toString.call(data) === '[object Object]';
};
exports['src::object.forEach'] = (() => {



    const isArray = include('is.array');
    const isObject = include('is.object');





    /**
     * 
     * 迭代整个对象数据
     * 
     * @import is.array
     * 
     * @import is.object
     * 
     * @param {Object} target
     * 
     * @param {Function} fn
     * 
     * @scoped
     * 
     */

    function main(target, fn) {

        fn(target);

        let keys = Object.keys(target);

        for (let key of keys) {

            let value = target[key];

            if (isArray(value)) {

                for (let item of value) {

                    if (main(item, fn) === false) {

                        return;
                    }
                }

            } else if (isObject(value)) {

                if (main(value, fn) === false) {

                    return;
                }
            }
        }
    }



    if (typeof main === 'function') {

        return function(target, fn) {

            return main.apply(undefined, arguments);
        };
    }

})();
exports['src::object.find.query'] = function(query, value) {



    const isObject = include('is.object.simple');
    const isFunction = include('is.function');
    const isString = include('is.string');
    const from = include('string.from.literal');





    /**
     * 
     * 生成或者获取一个查询函数
     * 
     * @import isObject from is.object.simple
     * 
     * @import is.function
     * 
     * @import is.string
     * 
     * @import from from string.from.literal
     * 
     * @param {String|Object} query 查询配置或者查询属性名称
     * 
     * @param {*} [value] 查询数据
     * 
     */


    if (isString(query)) {

        query = {
            [query]: value
        };
    }

    if (isObject(query)) {

        let keys = Object.keys(query),
            expression = [];

        for (let key of keys) {

            let value = query[key];

            if (isObject(value)) {

                if (value.hasOwnProperty('$exists')) {

                    if (value.$exists === true) {

                        expression.push(`('${key}' in target)`);

                    } else {

                        expression.push(`!('${key}' in target)`);
                    }
                }

            } else {

                expression.push(`target.${key} === ${from(query[key])}`);
            }
        }

        return new Function('target', `return ${expression.join('&&')};`);

    } else if (isFunction(query)) {

        return query;
    }
};
exports['src::object.find'] = function(target, query, value) {



    const isObject = include('is.object.simple');
    const isFunction = include('is.function');
    const isString = include('is.string');
    const from = include('string.from.literal');
    const forEach = include('object.forEach');
    const compile_query = include('object.find.query');





    /**
     * 
     * 在对象类型数据中进行查找并返回结果
     * 
     * @import isObject from is.object.simple
     * 
     * @import is.function
     * 
     * @import is.string
     * 
     * @import from from string.from.literal
     * 
     * @import forEach from object.forEach
     * 
     * @import compile_query from object.find.query
     * 
     * @param {Object} target 查询的对象类型数据
     * 
     * @param {SimpleObject|Function|String} query  查询信息
     * 
     * @param {*} [value] 查询属性匹配值
     * 
     * @return {Object|Void} 返回的查询结果，如果查询不到，则返回空
     * 
     * 
     */

    query = compile_query(query, value);

    let result;

    forEach(target, item => {

        if (query(item)) {

            result = item;

            return false;
        }

    });

    return result;
};
exports['src::object.find.all'] = function(target, query, value) {



    const isObject = include('is.object.simple');
    const isFunction = include('is.function');
    const isString = include('is.string');
    const from = include('string.from.literal');
    const forEach = include('object.forEach');
    const compile_query = include('object.find.query');





    /**
     * 
     * 在对象类型数据中进行查找并返回结果
     * 
     * @import isObject from is.object.simple
     * 
     * @import is.function
     * 
     * @import is.string
     * 
     * @import from from string.from.literal
     * 
     * @import forEach from object.forEach
     * 
     * @import compile_query from object.find.query
     * 
     * @param {Object} target 查询的对象类型数据
     * 
     * @param {SimpleObject|Function|String} query  查询信息
     * 
     * @param {*} [value] 查询属性匹配值
     * 
     * @return {Object[]} 返回的查询结果，如果查询不到，则返回空
     * 
     * 
     */

    query = compile_query(query, value);

    let result = [];

    forEach(target, item => {

        if (query(item)) {

            result.push(item);
        }

    });

    return result;
};

export default exports;